/**
 * Convex Hull 
 *
 * Complexity (Time): O(n log n)
 * Complexity (Space): O(n)
 */

typedef pair<double,double> dd;

// The three points are a counter-clockwise turn if cross > 0, clockwise if
// cross < 0, and collinear if cross = 0
double cross(dd a, dd b, dd c) {
  return (b.fi - a.fi) * (c.se - a.se) - (b.se - a.se) * (c.fi - a.fi);
}


// Find, among v, the points that form a convex hull
int convex_hull(vector<dd> &v) {
  int k = 0;
  vector<int> ans(v.sz * 2);

  // Sort points
  sort(v.begin(), v.end(), [](const dd &a, const dd &b) -> bool {
    return (a.fi == b.fi) ? (a.se < b.se) : (a.fi < b.fi);
  });

  // Uppermost part of convex hull
  for (int i = 0; i < v.sz; ++i) {
    while (k >= 2 && cross(v[ans[k - 2]], v[ans[k - 1]], v[i]) < 0) k--;
    ans[k++] = i;
  }

  // Lowermost part of convex hull
  for (int i = v.sz - 2, t = k + 1; i >= 0; --i) {
    while (k >= t && cross(v[ans[k - 2]], v[ans[k - 1]], v[i]) < 0) k--;
    ans[k++] = i;
  }

  // The ans vector contains the indices (relative to the sorted vector!) of 
  // the points belonging to the convex hull
  ans.resize(k);

  // Remove duplicates
  sort(all(ans));
  ans.erase(unique(all(ans)), ans.end());

  // Returns amount of point in the convex hull
  return k - 1;
}
