/**
 * Articulations and Bridges (Tarjan)
 *
 * Complexity (Time): O(n + m)
 * Complexity (Space): O(n + m)
 */

vector<int> graph[MAX];

int cont[MAX], parent[MAX];
int low[MAX], L[MAX];
vector<ii> bridges;
vector<int> articulations;

// Finds all articulations and bridges in the graph
void dfs(int x) {
  int child = 0;
  cont[x] = 1;

  for (auto i : graph[x]) {
    if (!cont[i]) {
      child++;
      parent[i] = x;

      low[i] = L[i] = L[x] + 1;
      dfs(i);
      low[x] = min(low[x], low[i]);

      if ((parent[x] == -1 && child > 1) || (parent[x] != -1 && low[i] >= L[x]))
        articulations.pb(x);

      if (low[i] > L[x])
        bridges.pb(ii(x, i));

    } else if (parent[x] != i) {
      low[x] = min(low[x], L[i]);
    }
  }
}


void tarjan(int v) {
  memset(parent, -1, sizeof parent);
  dfs(v);

  // Remove duplicates for articulations
  sort(all(articulations));
  articulations.erase(unique(all(articulations)), articulations.end());
}
